#include <tigcclib.h>
#include "D:\Documents\TIGCC\ExtGraph-master\lib\extgraph.h"


// type definitions
typedef struct {
	float X, Y;
} Vector2;

// screen buffer
LCD_BUFFER dark, light, screen_dark, screen_light;

// sprites
typedef struct {
	unsigned long int *mask, *dark, *light;
	unsigned short height;
} Sprite;
#define BIRD_HEIGHT 12
#define BIRD_WIDTH  17
#define PIPE_HEIGHT 128
#define PIPE_WIDTH  32
static unsigned long int bird_mask[BIRD_HEIGHT]  = {~0x03F00000,~0x0FF80000,~0x1FFC0000,~0x7FFE0000,~0xFFFE0000,~0xFFFE0000,~0xFFFF0000,~0x7FFF8000,~0x3FFF0000,~0x3FFF0000,~0x1FFE0000,~0x07C00000};
static unsigned long int bird_dark[BIRD_HEIGHT]  = { 0x03F00000, 0x0C480000, 0x10840000, 0x788A0000, 0x848A0000, 0x82420000, 0x823F0000, 0x44408000, 0x38BF0000, 0x20410000, 0x183E0000, 0x07C00000};
static unsigned long int bird_light[BIRD_HEIGHT] = { 0x03F00000, 0x0FC80000, 0x1F840000, 0x7F8A0000, 0xFF8A0000, 0xFFC20000, 0xFFFF0000, 0x7FFF8000, 0x3FFF0000, 0x3FFF0000, 0x1FFE0000, 0x07C00000};
static unsigned long int bird2_mask[BIRD_WIDTH]  = {~0x07000000,~0x0F800000,~0x3F800000,~0x7FC00000,~0x7FE00000,~0xFFE00000,~0xFFF00000,~0xFFF00000,~0xFFF00000,~0xFFF00000,~0x7FF00000,~0x7FF00000,~0x7FE00000,~0x7FC00000,~0x7F800000,~0x3C000000,~0x08000000};
static unsigned long int bird2_dark[BIRD_WIDTH]  = { 0x07000000, 0x08800000, 0x30800000, 0x50C00000, 0x50A00000, 0x89200000, 0x86100000, 0x80100000, 0x91D00000, 0xAA300000, 0x54100000, 0x54100000, 0x55A00000, 0x54400000, 0x57800000, 0x34000000, 0x08000000};
static unsigned long int bird2_light[BIRD_WIDTH] = { 0x07000000, 0x0F800000, 0x3F800000, 0x7FC00000, 0x7FE00000, 0xFFE00000, 0xFFF00000, 0xFFF00000, 0xFFF00000, 0xFE300000, 0x7C100000, 0x7C100000, 0x7DA00000, 0x7C400000, 0x7F800000, 0x3C000000, 0x08000000};
static unsigned long int pipe0_dark[PIPE_HEIGHT], pipe0_light[PIPE_HEIGHT];
static unsigned long int pipe1_dark[PIPE_HEIGHT], pipe1_light[PIPE_HEIGHT];
static unsigned long int pipe2_dark[PIPE_HEIGHT], pipe2_light[PIPE_HEIGHT];
static unsigned long int pipe3_dark[PIPE_HEIGHT], pipe3_light[PIPE_HEIGHT];
static unsigned long int pipe4_dark[PIPE_HEIGHT], pipe4_light[PIPE_HEIGHT];
static unsigned long int pipe5_dark[PIPE_HEIGHT], pipe5_light[PIPE_HEIGHT];
static unsigned long int pipe6_dark[PIPE_HEIGHT], pipe6_light[PIPE_HEIGHT];
static unsigned long int black[128];
Sprite Bird[2] = {{bird_mask,  bird_dark,  bird_light , BIRD_HEIGHT},
                  {bird2_mask, bird2_dark, bird2_light, BIRD_WIDTH }},
       Pipe[7] = {{NULL, pipe0_dark, pipe0_light, PIPE_HEIGHT},
                  {NULL, pipe1_dark, pipe1_light, PIPE_HEIGHT},
                  {NULL, pipe2_dark, pipe2_light, PIPE_HEIGHT},
                  {NULL, pipe3_dark, pipe3_light, PIPE_HEIGHT},
                  {NULL, pipe4_dark, pipe4_light, PIPE_HEIGHT},
                  {NULL, pipe5_dark, pipe5_light, PIPE_HEIGHT},
                  {NULL, pipe6_dark, pipe6_light, PIPE_HEIGHT}};

// main function prototypes
void draw (void);
void clrdraw (void);

void _main(void)
{
	if (!GrayOn ()) // turn grayscale on
		return;
	
	GrayClearScreen();
	
	// initialize
	{
		#define RIM_SIZE 12*sizeof(long)
		unsigned long int pillar_dark[]  = {0x20000004},
		                  pillar_light[] = {0x3E0007FC},
		                  rim_dark[]     = {0xFFFFFFFF,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0x80000001,0xFFFFFFFF},
		                  rim_light[]    = {0xFFFFFFFF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFC0003FF,0xFFFFFFFF};
		Sprite Pillar = {NULL, pillar_dark, pillar_light, 1},
		       Rim    = {NULL, rim_dark,    rim_light   , 12};
		
		short pipe, y;
		for (pipe = 0; pipe < 7; pipe++) {
			y = 0;
			while (y < pipe*8+6) {
				Pipe[pipe].dark[y] = Pillar.dark[0];
				Pipe[pipe].light[y] = Pillar.light[0];
				y++;
			}
			memcpy (&Pipe[pipe].dark[y], &Rim.dark[0], RIM_SIZE);
			memcpy (&Pipe[pipe].light[y], &Rim.light[0], RIM_SIZE);
			y += 52;
			memcpy (&Pipe[pipe].dark[y], &Rim.dark[0], RIM_SIZE);
			memcpy (&Pipe[pipe].light[y], &Rim.light[0], RIM_SIZE);
			y += 12;
			while (y < LCD_HEIGHT) {
				Pipe[pipe].dark[y] = Pillar.dark[0];
				Pipe[pipe].light[y] = Pillar.light[0];
				y++;
			}
		}
		
		for (y = 0; y < 128; y++) {
			black[y] = (long)0xFFFFFFFF;
		}
		
		GraySprite32_OR (0, 0, 128, black, black, screen_light, screen_dark);
		GraySprite32_OR (208, 0, 128, black, black, screen_light, screen_dark);
	}
	
	float birdMotY = 0;
	Vector2 birdPos    =  {80 ,30};
	Vector2 pipePos[4] = {{80 , 0},
	                      {160, 0},
	                      {240, 0}};
	
	void *kbq = kbd_queue ();
	unsigned short key;
	
	#define SPEED 12
	#define TIMER 5
	short old_rate = PRG_getRate ();
	PRG_setRate (0);
	OSFreeTimer (TIMER);
	
	enum PipeTypes {PIPE_0, PIPE_1, PIPE_2, PIPE_3, PIPE_4, PIPE_5, PIPE_6, NO_PIPE};
	const short pipe_min = 65, pipe_max = 95; // constants for collision detection
	const float boost = 6, gravity = 1, speed = 2; // constants for game logic
	{
		restart:;
		
		short currentPipe, type, position; // temporary variables
		char pipeType[3] = {NO_PIPE, NO_PIPE, NO_PIPE}; // current pipes stored in an array
		signed char dead = 0, loop = 1;
		signed short score = -2;
		
		while (loop) {
			OSRegisterTimer (TIMER, SPEED);
			{ // draw screen
				clrdraw (); // clear screen
				
				// draw sprites
				for (currentPipe = 0; currentPipe < 3; currentPipe++) {
					type = pipeType[currentPipe];
					if (type != NO_PIPE)
						GraySprite32_OR ((short)pipePos[currentPipe].X, (short)pipePos[currentPipe].Y, PIPE_HEIGHT, Pipe[type].light, Pipe[type].dark, light, dark);
				}
				GraySprite32_MASK ((short)birdPos.X, (short)birdPos.Y, Bird[dead].height, Bird[dead].light, Bird[dead].dark, Bird[dead].mask, Bird[dead].mask, light, dark);
				char buffer[10];
				sprintf(buffer, "%d", score);
				if (score >= 0) GrayDrawStr2B(0, 0, buffer, A_XOR, light, dark);
				
				draw ();
			}
			
			if (!OSdequeue (&key, kbq)) { // checks if key is pressed
				if (key == KEY_ENTER) {
					if (!dead)
						birdMotY = -boost;
					else
						goto restart;
				} else if (key == KEY_ESC)
					loop = 0;
			}
			
			if (!dead) {
				for (currentPipe = 0; currentPipe < 3; currentPipe++) {
					position = (pipePos[currentPipe].X -= speed);
					type = (pipeType[currentPipe]);
					if (position < 0) {
						randomize();
						pipeType[currentPipe] = random(7);
						pipePos[currentPipe].X = 240;
						score++;
					} else if ((position > pipe_min && position < pipe_max) && type != NO_PIPE) {
						if (birdPos.Y < type*8+18 || birdPos.Y > type*8+58)
							dead = 1;
					}
				}
			}
			
			birdPos.Y += birdMotY;
			if (birdPos.Y > LCD_HEIGHT - Bird[dead].height)
				birdPos.Y = LCD_HEIGHT - Bird[dead].height;
			if (birdPos.Y < 0)
				birdPos.Y = 0;
			birdMotY += gravity;
			
			while (!OSTimerExpired(TIMER));
		}
	}
	
	PRG_setRate(old_rate);
	
	GrayOff();
}


void draw (void)
{
	FastCopyScreen(dark, GetPlane(DARK_PLANE));
	FastCopyScreen(light, GetPlane(LIGHT_PLANE));
}

void clrdraw (void)
{
	FastCopyScreen(screen_dark, dark);
	FastCopyScreen(screen_light, light);
}
